package Puma::Core;

=copyright

	Puma - Perl Universal Markup
	Copyright (c) 2001, Ingmar Ellenberger.  All rights reserved.
	Software is distributed AS IS.  No warranties, expressed or implied.

	This license is temporary, pending the selection of an appropriate
	open-source license.

=description

	Core.pm - An abstract base class for all Puma classes.

	The Core package provides a unified construction mechanism with full
	inheritance support for all subclasses.

	Subclasses should not define a 'new' constructor.  If processing is required
	as a part of the construction process, the following code should be
	implemented:

		sub construct {
			my $self = shift;

			my %arg = $self->Puma::Core::construct(@_);
			# or alternately: my %arg = $self->SUPER::construct(@_);

			# ... insert your code here ...

			return %arg;
		}

	Note that the Core class presumes that the class constructor will be called
	using named (not positional) parameters.

=cut

use strict;

# === Object Construction ===================================================
# --- constructor ---
sub new {
	my $this = shift;
	my $self = bless {}, ref($this) || $this;

	# TODO: establish bullet proof error handling
	eval {
		# --- call constructor ---
		my @res = $self->construct(@_);
	};
	warn $@ if $@;
	
	return $self;
}

# --- construction interface ---
sub construct { my $self = shift; return @_; }


# -- a helper for processing named parameters ---
sub stripArgs {
	my ($self, $args, @arglist) = @_;

	foreach my $arg (@arglist) {
		if (exists $args->{$arg} && defined $args->{$arg}) {
			$self->{$arg} = $args->{$arg};
			delete $args->{$arg};
		}
	}
}
1;
