package Puma::Parser::SP;
use ibase Puma::Parser;

=copyright

	Puma - Perl Universal Markup
	Copyright (c) 2001, Ingmar Ellenberger.  All rights reserved.
	Software is distributed AS IS.  No warranties, expressed or implied.

	This license is temporary, pending the selection of an appropriate
	open-source license.

=description

	SP.pm - Server Pages extension if the Puma Parser

=cut

use strict;

# --- standard constructor ---
sub construct {
	my $self = shift;

	# --- set the rules for extracting valid labels ---
	return $self->SUPER::construct(@_,
		LabelRules => '[^<\s]*:',
		IncludeTag => ':include',
		Handlers => {
			':include' => \&include,
		},
	);
}

# --- turn the hash into eval'able code ---
sub codify {
	my ($self, $ptree) = @_;

	my $code = '';	#!/usr/bin/perl -w\n\n";
	$code .= "use strict;\n\n";
	$code .= $self->_walk(@$ptree) if $ptree;
#print 'SP::codify '. Data::Dumper->Dump([$ptree], ['ptree']) ."\n";
#print "<pre>". Data::Dumper->Dump([$code], ['code']) ."</pre><br>\n";
	return $code;
}

# --- evaluate the resulting code ---
sub evaluate {
	my $self = shift;

	# return "hello!\n";
	return eval(shift);
}

sub write {
	my $self = shift;
}

# --- private method called recursively to walk the tree ---
sub _walk {
	my ($self, $tagname, $params, @ptree) = @_;

	# --- split tagname ---
	$tagname =~ /([^:]*):(.*)/;
	my ($prefix, $name) = ($1 || '', $2 || '');

	my $retval;
	# --- process special commands ---
	unless ($prefix) {
		return "$params->{code}\n" unless $name; # in-line code
		return qq[print($params->{eval});\n] if $name eq '='; # evaluations

		if ($name eq 'app') {
			$retval .= "use $params->{module}; my \$$params->{prefix} = new $params->{module};\n";
		} elsif ($name eq 'lib') {
			$retval .= "use lib '$params->{path}';\n\n";
		} elsif ($name ne 'parser') {
			$retval .= qq[print("<b>Unknown Tag '$tagname'</b><br>\\n");\n];
		}
	} else {
		$retval .= '$'. $prefix ."->_render('$name', {";
		foreach my $key (keys %$params) {
			$retval .= " $key => qq[$params->{$key}],";
		}
		$retval .= " }, sub {\n";
	}

	# --- walk the tag array ---
if (@ptree) {
   foreach my $child (@ptree) {
		if (ref $child) {
			$retval .= $self->_walk(@{$child});
		} else {
			$retval .= 'print qq['. $child ."];\n";
		}
	}
}

	$retval .= "});\n" if $prefix;
	return $retval;
}

=method

	include - Compile time server-side include

	Added as a part of the new Handler mechanism for the parser, this method
   allows server-side includes whenever the ':include' label is found.

=cut

# TODO: make a case insensitive hash class/tie?
use Puma::Util qw(lckey);  # case insensitive hash keys

sub include {
	my ($self, $tag) = @_;

	my $filename = lckey($tag->{params}, 'filename');
	return "no filename" unless $tag->{params} && $filename;
	my $parser = $self->new();  # use $self just in case the class has been extended
	my $ptree = $parser->parseFile($filename);
	splice @{$ptree}, 0, 2;

	return @{$ptree};
}


1;
