package Puma::Parser::XML;
use ibase Puma::Parser;

=copyright

	Puma - Perl Universal Markup
	Copyright (c) 2001, Ingmar Ellenberger.  All rights reserved.
	Software is distributed AS IS.  No warranties, expressed or implied.

	This license is temporary, pending the selection of an appropriate
	open-source license.

=description

	XML.pm - XML extension if the Puma Parser

=cut

use strict;

# --- force parser into XML mode ---
sub construct {
	my ($self, %args) = @_;

	$self->stripArgs(\%args, qw(XML XMLHandlers Object));
	$args{Text} = $self->{XML} if $self->{XML};

	return $self->SUPER::construct(%args);
}

# --- handler accessor ---
sub callXMLHandler {
	my ($self, $handlername) = (shift, shift);

	return unless $self->{XMLHandlers}->{$handlername};

	my $handler = $self->{XMLHandlers}->{$handlername};
	if ($self->{Object}) {
		&{$handler}($self->{Object}, @_);
	} else {
		&{$handler}(@_);
	}
}

# --- overloaded method to parse the XMLish file ---
sub parse {
	my ($self, %args) = @_;

	$self->stripArgs(\%args, qw(XML XMLHandlers Object));
	$args{Text} = $self->{XML} if $self->{XML};

	# --- call parent parser ---
	my $ptree = $self->SUPER::parse(%args);
	return unless $ptree;

	# --- call 'Init' handler ---
	$self->callXMLHandler('Init');

	# --- walk the tree ---
	$self->_walk(@$ptree);

	# --- call 'Final' handler ---
	$self->callXMLHandler('Final');

	return $ptree;
}

# --- private method called recursively to walk the tree ---
sub _walk {
	my ($self, $tagname, $params, @ptree) = @_;

	# --- call 'Start' handler ---
	$self->callXMLHandler('Start', $tagname, %{$params});

	# --- walk the tag array ---
   foreach my $child (@ptree) {
		if (ref $child) {
			# --- recurse ---
			$self->_walk(@$child);
		} else {
			# --- call 'Char' handler ---
			$self->callXMLHandler('Char', $child);
		}
	}

	# --- call 'End' handler ---
	$self->callXMLHandler('End', $tagname);
}

=docs

   This is a weak XMLish implementation of the OpenePSP parser.
	It is not nearly as strict as XML::Parser, but it is not as weak as HTML.
	Excellent for configuration files.

=cut

1;
