package Puma::RegEx;
use ibase Exporter;

=copyright

	Puma - Perl Universal Markup
	Copyright (c) 2001, Ingmar Ellenberger.  All rights reserved.
	Software is distributed AS IS.  No warranties, expressed or implied.

	This license is temporary, pending the selection of an appropriate
	open-source license.

=description

	RegEx.pm - Regular Expression Generator

	This package was created to remove some of the here docs from the code.

   Placing many of the more complex regular expressions in this package has
   the advantage of making the original code more readable by removing
   heredocs while also allowing room for more extensive documentation of the
   regular expressions.

=cut

@EXPORT_OK = qw(getRegEx);


sub getRegEx {
	my $name = lc(shift);

	my $regex = undef;
	SWITCH: {

		# --- regex for detecting and extracting tags ---
		'tagparser' eq $name && do {
			# TODO: change this regex because it takes a LONG time to run when there
			#       are mismatched quotes
			my $labelrules = shift;
			$labelrules = $labelrules ? '(?:'. $labelrules .')' : '';
			$regex =
				'(.*?)(<' .                     # anything up to a tag + the tag start
				$labelrules .                   # add the label rules
<<'REGEX';
				(?:"(?:[^"\\]*(?:\\.[^"\\]*)*)" # doublequoted string
				  |'(?:[^'\\]*(?:\\.[^'\\]*)*)' # singlequoted string
				  |`(?:[^`\\]*(?:\\.[^`\\]*)*)` # backquoted string
				  |[^"'`>]*)                    # or anything but a string or tag close
				*>)                             # end the tag
REGEX
			$regex = qr/\G$regex/sx;  # '/x' filters comments and whitespace
			last;
		};


		# --- regex for splitting up the tags ---
		'tagsplitter' eq $name && do {
			$regex =
<<'REGEX';
				([\w.-]+)\s*=\s*              # get key and '='
				(?:"([^"\\]*(?:\\.[^"\\]*)*)" # doublequoted value
				  |'([^'\\]*(?:\\.[^'\\]*)*)' # singlequoted value
				  |(`[^`\\]*(?:\\.[^`\\]*)*`) # backquoted value (keep quotes)
				  |([^\s>]*))\s*              # unquoted value
REGEX
			$regex = qr/\G$regex/sx;  # '/x' filters comments and whitespace
			last;
		};
	};

	return $regex;
}

1;
