package Puma::State::DDState;

=copyright

	Puma - Perl Universal Markup
	Copyright (c) 2001, Ingmar Ellenberger.  All rights reserved.
	Software is distributed AS IS.  No warranties, expressed or implied.

	This license is temporary, pending the selection of an appropriate
	open-source license.

=description

	DDState.pm - A simple object implemented using Data::Dumper

=cut

use Puma::Util qw(readfile writefile);

use strict;

# TODO: build config object to be passed in
#    $self->{_location} - db, path, whatever.
#    $self->{_context}  - application name
#    $self->{_uid}      - unique session key

# --- object contructor ---
sub new {
	# --- a hellofan effort to get a blessed obj ref to a tied hash! ---
	my $this = shift;                    # get classname or ref to instantiated object
	my $class = ref($this) || $this;     # get classname from ref if necessary
	my %hashtie = ();                    # create a hash to tie
	tie %hashtie, $class;                # tie hash to class (for hash tie)
	my $self = bless \%hashtie, $class;  # bless ref to tied hash into class (for object)

	# --- Pull in remainind params ---
	my %args = @_;
	$self->{_location} = $args{Location} if $args{Location};
	$self->{_context}  = $args{Context}  if $args{Context};
	$self->{_uid}      = $args{UID}      if $args{UID};

	$self->{_content} = {};

	return $self;
}

sub load {
	my $self = shift;
	eval readfile($self->filename);
}

sub save {
	my $self = shift;

	use Data::Dumper;
	my $dump = new Data::Dumper([$self->{_content}], ['self->{_content}']);
	$dump->Indent(0);
	$dump->Purity(1);

	writefile($self->filename, $dump->Dump);
}

sub filename {
	my $self = shift;
	return $self->{_location} .'.'. $self->{_context} .'.'. $self->{_uid};
	# --- this is the real filename! ---
	# return $self->{_location} .'/'. $self->{_context} .'/'. $self->{_uid};
}

sub lasttouch {
	# TODO: return epoch seconds since last access
	return 0;
}

# === hash tie stuff ========================================================

# --- list of reserved elements used internally ---
my $reserved = qr/_(content|context|location|uid)/s;

# --- hash constructor ---
sub TIEHASH {
	my ($this, $hash) = (shift, shift || {});
	return bless $hash, ref($this) || $this;
}

sub STORE {
	my ($self, $key, $value) = @_;
	return $self->{$key} = $value if $key =~ /$reserved/;
	return $self->{_content}->{$key} = $value;
}

sub FETCH    {
	my ($self, $key) = @_;
	return $self->{$key} if $key =~ /$reserved/;
	return $self->{_content}->{$key};
}

sub FIRSTKEY { my $a = scalar keys %{$_[0]->{_content}}; each %{$_[0]->{_content}} }
sub NEXTKEY  { each %{$_[0]->{_content}} }
sub EXISTS   { exists $_[0]->{_content}->{$_[1]} }
sub DELETE   { delete $_[0]->{_content}->{$_[1]} }
sub CLEAR    { %{$_[0]->{_content}} = () }1;

1;
