package Puma::Util;
use ibase Exporter;

=copyright

	Puma - Perl Universal Markup
	Copyright (c) 2001, Ingmar Ellenberger.  All rights reserved.
	Software is distributed AS IS.  No warranties, expressed or implied.

	This license is temporary, pending the selection of an appropriate
	open-source license.

=description

	Util.pm - Various stand-alone utilities

	Note that Util.pm does not necessarly have any relationship with the classes
	in the Puma/Util directory.

=cut

@EXPORT_OK = qw(readfile writefile writelog lckey);

use strict;

# TODO: change these functions to use dynamic handles

# === Read a file, return the contents ======================================
sub readfile {
	my $filename = shift;

	return undef unless $filename;

	# --- undefine system record separator to read file in one big chunk ---
	my $recordseparator = $/; undef $/;

	# --- open file and read contents ---
	open INFILE_Puma_Util_readfile, $filename or return undef;
	my $content = <INFILE_Puma_Util_readfile>;
	close INFILE_Puma_Util_readfile;

	# --- set system record separator back to original value ---
	$/ = $recordseparator;

	# --- return file contents ---
	return $content;
}

# === Write (create/overwrite) a file =======================================
sub writefile {
	my $filename = shift;
	my $content = shift || '';

	return unless $filename;

	open OUTFILE_Puma_Util_writefile, ">$filename" or return;
	print OUTFILE_Puma_Util_writefile $content;
	close OUTFILE_Puma_Util_writefile;
}

# --- a simple utility to write messages to a log file ---
sub writelog {
	my $filename = '/tmp/error_log';
	open ERRFILE_Puma_Util_writelog, ">>$filename";
	chmod 0666, $filename;
	print ERRFILE_Puma_Util_writelog localtime(time) .' [message] ';
	print ERRFILE_Puma_Util_writelog @_;
	print ERRFILE_Puma_Util_writelog "\n";
	close ERRFILE_Puma_Util_writelog;
}

# TODO: make a case insensitive hash class/tie?
# --- get a case insensitive key from a hash ---
sub lckey {
   my ($hash, $key) = @_;
	return undef unless ref $hash && defined $key;

	$key = lc($key);
	foreach my $hashkey (keys %{$hash}) {
		return $hash->{$hashkey} if lc($hashkey) eq $key;
	}

	return undef;
}

1;
