package Puma::Util::Config;
use ibase Puma::Parser::XML;

=copyright

	Puma - Perl Universal Markup
	Copyright (c) 2001, Ingmar Ellenberger.  All rights reserved.
	Software is distributed AS IS.  No warranties, expressed or implied.

	This license is temporary, pending the selection of an appropriate
	open-source license.

=description

	Config.pm - A configuration file parser based on Puma::Parser::XML

=cut

use Puma::Util qw(readfile);

use strict;

=TODO
	- eventually standardize the naming convention for packages
		- use package name and translate to appname
		- use appname as the standard?
=cut

sub parse {
   my $self = shift;

	my $ptree = $self->SUPER::parse(@_,
		XMLHandlers => {
			Init  => \&init,
			Start => \&start,
			Char  => \&char,
			End   => \&end,
			Final => \&final,
		},
		Object => $self,
	);

	return $ptree;
}

# === XML Handlers ==========================================================

sub init {
	my $self = shift;
	$self->{stack} = [];
	$self->{depth} = 0;
}

sub start {
	my ($self, $tagname) = (shift, shift);
	my $depth = $self->{depth}++;

	$self->{stack}->[$depth]->{TaGlAbEl} = $tagname;
	my %args = @_;
	foreach my $key (keys %args) {
		$self->addParam($key, $args{$key});
	}
}

sub char {
	my ($self, $content) = (shift, shift);
	return unless $self->{depth} > 0;

	# --- trim out leading and trailing whitespace ---
	$content =~ s/^\s*(.*?)\s*$/$1/g;
	return unless $content;
	$self->addParam($content, '');
}

sub end {
	my $self = shift;
	return unless $self->{depth} > 0;
	my $depth = --$self->{depth};

	my $key = $self->{stack}->[$depth]->{TaGlAbEl};
	my $value = $self->{stack}->[$depth + 1];

	$value = (keys %{$value})[0] if keys %{$value} == 1 && (values %{$value})[0] eq '';

	$self->{stack}->[$depth]->{$key} = $value;

	delete $self->{stack}->[$depth]->{TaGlAbEl};
	$self->{stack}->[$depth + 1] = {};
}

sub final {
	my $self = shift;
	$self->{hash} = $self->{stack}->[0]->{':parser'};
}

# === XML Handler Utilities ==================================================

sub addParam {
	my ($self, $key, $value) = @_;
	$self->{stack}->[$self->{depth}]->{$key} = $value;
}


1;
