package ibase;

=description

	This package fixes a problem with the core Perl base.pm module which, under
	certain circumstances, may incorrectly detect whether or not a package has
	been loaded.  It is a direct replacemant for the core Perl base.pm module
	and should be kept current with it.

	The only difference there should be between the two packages is that the
	line:

		unless (defined %{"$base\::"}} {

	from base.pm should be changed to:

		unless (defined %{"$base\::"} && exists ${"$base\::"}{BEGIN}) {

	for ibase.pm.

=cut

sub import {
	my $class = shift;
	my $fields_base;

	foreach my $base (@_) {
		unless (defined %{"$base\::"} && exists ${"$base\::"}{BEGIN}) {
			eval "require $base";
			# Only ignore "Can't locate" errors from our eval require.
			# Other fatal errors (syntax etc) must be reported.
			die if $@ && $@ !~ /^Can't locate .*? at \(eval /;
			unless (defined %{"$base\::"}) {
				require Carp;
				Carp::croak("Base class package \"$base\" is empty.\n",
					"\t(Perhaps you need to 'use' the module ",
					"which defines that package first.)");
			}
		}

		# A simple test like (defined %{"$base\::FIELDS"}) will
		# sometimes produce typo warnings because it would create
		# the hash if it was not present before.
		my $fglob;
		if ($fglob = ${"$base\::"}{"FIELDS"} and *$fglob{HASH}) {
			if ($fields_base) {
				require Carp;
				Carp::croak("Can't multiply inherit %FIELDS");
			} else {
				$fields_base = $base;
			}
		}
	}
	my $pkg = caller(0);
	push @{"$pkg\::ISA"}, @_;
	if ($fields_base) {
		require fields;
		fields::inherit($pkg, $fields_base);
	}
}

1;
